/*
    Copyright (C) 1998, 1999  Michael Ravits

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "include/newplayer.h"
#include "include/stuff.h"
#include "include/screen.h"
#include "include/player_newplayer.h"
#include "include/playlist.h"


playlist::playlist(void)
{
	songs_num = 0;
	play_list = new char*[block_size];
	list_size = block_size;
	playing = FALSE;
	position = 0;
	path_and_file = curr_path = some_str = NULL;
	list_play_method = ALPHA;
}

playlist::~playlist(void)
{
	free_list();
}

void playlist::add_to_playlist(char *new_song)
{
	if(playing)
		return;

	if(!(songs_num % block_size)&&(songs_num != 0))
	{
		char	**temp;
	
		temp = new char*[songs_num + block_size];
		memcpy((void *)temp, (void *)play_list, sizeof(char *) * songs_num);

		delete	play_list;
		play_list = temp;
		list_size += block_size;
	}

	if(new_song[0] != '/')
	{
		give_path_and_file(new_song);
		play_list[songs_num] = new char[strlen(path_and_file) + 1];
		strcpy(play_list[songs_num], path_and_file);
	}
	else
	{
		play_list[songs_num] = new char[strlen(new_song) + 1];
		strcpy(play_list[songs_num], new_song);
	}

	songs_num++;
}

char *find_file_in_string(char *bill_gates)
{
	static	char	temp[4096], temp2[4096];
	int	i, idx;
	
	i   = strlen(bill_gates) - 1;
	idx = 0;
	
	while(bill_gates[i] != '/')
	{
		temp[idx] = bill_gates[i];	
		idx++; i--;
	}

	temp[idx] = '\0';

	idx = strlen(temp) - 1;
	i = 0;

	while(idx >= 0)
	{
		temp2[i] = temp[idx];
		i++;
		idx--;
	}
	
	temp2[strlen(temp)] = '\0';
	
	return temp2;
}

int playlist::in_playlist(char	*song_name)
{
	int	i, temp;
	char	*tmp_str;
	
	for(i = 0; i < songs_num; i++)
	{	
		tmp_str = find_file_in_string(play_list[i]);
		temp = strcmp(tmp_str, song_name);
		if(!temp)
			return i;
	}
	
	return -1;
}

void playlist::remove_from_playlist(int song_num)
{
	int	i;
	char	**temp;

	if(playing)
		return;
	
	temp = new char*[list_size];

	for(i = 0; i < song_num; i++) 
	{
		temp[i] = new char[strlen(play_list[i]) + 1];
		strcpy(temp[i], play_list[i]);
	}

	for(i = song_num; i < (songs_num - 1); i++)
	{
		temp[i] = new char[strlen(play_list[i + 1]) + 1];
		strcpy(temp[i], play_list[i + 1]);
	}

	free_list();
	
	play_list = temp; 

	songs_num--;
}

void playlist::remove_all_from_playlist(void)
{
	int	i;

	for(i = 0; i < songs_num; i++)
	{
		delete play_list[i];
		play_list[i] = NULL;
	}

	songs_num = 0;
}

int cmp_func(const void *sng1, const void *sng2)
{
	int i = strcmp(*(char **)sng1, *(char **)sng2); 
	return i;
}

void playlist::sort_alpha(void)
{

	qsort((void *)play_list, songs_num, sizeof(char *), cmp_func);
}

void playlist::goto_first_song_in_list(class player &pl, bool cont)
{
	position = -1;
	play_the_list(pl, cont);
}

void playlist::goto_last_song_in_list(class player &pl, bool cont)
{
	position = songs_num - 2;
	play_the_list(pl, cont);
}

void playlist::play_prev_song_in_list(class player &pl, bool cont)
{
	if(position != 0)
		position -= 2;
	else
		--position;
	play_the_list(pl, cont);
}

void playlist::play_the_list(class player &pl, bool cont)
{
	if(list_play_method == ALPHA)
		play_the_list_alpha(pl, cont);
	else
		play_the_list_random(pl);
}

void playlist::play_the_list_alpha(class player &pl, bool cont)
{
	if((!playing) && (!cont))
	{
		if(songs_num == 0)
			return;
		sort_alpha();
		playing = TRUE;
		position = 0;
		pl.play(play_list[position]);
	}
	else
		if((cont)&&(playing))
		{
			if(position != (songs_num-1))
			{
				position++;
				pl.play(play_list[position]);
			}
			else
				playing = FALSE;
		}
}

int playlist::random_song(void)
{
	return (int)(((float)songs_num*rand()/(RAND_MAX+1.0)));
}

void playlist::play_the_list_random(class player &pl)
{
	int	i;

	if(!playing)
	{
		playing = TRUE;
		i = random_song();
		pl.play(play_list[i]);
	}
	else
		pl.play(play_list[random_song()]);
}

void playlist::free_list(void)
{
	if(play_list)
	{
		for(int i = 0; i < songs_num; i++)
		{
			delete	play_list[i];
			play_list[i] = NULL;
		}
		
		delete play_list;
		play_list = NULL;
	}
}

bool playlist::is_playing(void)
{
	return playing;
}

void playlist::reset_playing(void)
{
	position = 0;
	playing = FALSE;
}

void playlist::give_path_and_file(char *curr_file)
{
	if(curr_path)
		delete curr_path;
	if(path_and_file)
		delete path_and_file;
	
	curr_path = get_current_dir_name();
	path_and_file = new char [strlen(curr_path) + strlen(curr_file) + 2];
	strcpy(path_and_file, curr_path);
	strcat(path_and_file, "/");
	strcat(path_and_file, curr_file);
}

void playlist::set_play_method(ListPlay lp_method)
{
	list_play_method = lp_method;
}

void playlist::toggle_play_method(void)
{
	if(list_play_method == ALPHA)
		set_play_method(RANDOM);
	else
		set_play_method(ALPHA);
}

ListPlay playlist::rplay_method(void)
{
	return list_play_method;
}

void playlist::save_playlist(screen &scr, stuff &st, player &pl)
{
	FILE	*save_file;
	WINDOW	*in_window;
	char	fn_string[COLS - 4 + 5];

	in_window = scr.rinput_window();
	input_window_active = TRUE;
	scr.draw_screen(st, *this, pl);

	wattrset(in_window, A_BOLD|COLOR_PAIR(1));
	werase(in_window);
	box(in_window, 0, 0);
	
	//Getting file name
	
	echo();
	wmove(in_window, 1, 1);
	wgetnstr(in_window, fn_string, COLS - 4);
	input_window_active = FALSE;
	noecho();

	scr.draw_screen(st, *this, pl);

	//Adding `.npl' appendix
	
	strcat(fn_string, ".npl");
	if(!strcmp(fn_string, ".npl"))
		return;
	if(!(save_file = fopen(fn_string, "w")))
	{
		st.print_error("Hmpf, can't open playlist file for saving.");
		return;
	}
	
	sort_alpha();
	
	//Saving
	
	for(int i = 0; i < songs_num; i++)
	{
		fprintf(save_file, "%s\n", play_list[i]);
	}
	
	fflush(save_file);
	fclose(save_file);
}

void playlist::load_playlist(char *filename)
{
	FILE	*load_file;
	stuff	white_stuff;
	char	tmp_str[4096];
	
	remove_all_from_playlist();
	
	if(!(load_file = fopen(filename, "r")))
	{
		white_stuff.print_error("Can't open file to load playlist.");
		exit(-1);
	}

	while(!feof(load_file))
	{
		tmp_str[0] = '\0';
		fgets(tmp_str, 4096, load_file);
		tmp_str[strlen(tmp_str) - 1] = '\0';
		if(tmp_str[0] != '\0')
			add_to_playlist(tmp_str);
	}
}

bool playlist::is_input_window_active(void)
{
	return input_window_active;
}
